@@ -11,6 +11,7 @@
  * @brief Server Thread Definitions
  */
 
+#include <config.h>
 #include <libgearman-server/common.h>
 
 #define GEARMAN_CORE
@@ -133,7 +134,7 @@ void gearman_server_thread_free(gearman_server_thread_st *thread)
   {
     con= thread->free_con_list;
     thread->free_con_list= con->next;
-    gearmand_crazy("free");
+    gearmand_debug("free");
     free(con);
   }
 
@@ -141,7 +142,7 @@ void gearman_server_thread_free(gearman_server_thread_st *thread)
   {
     packet= thread->free_packet_list;
     thread->free_packet_list= packet->next;
-    gearmand_crazy("free");
+    gearmand_debug("free");
     free(packet);
   }
 
