@@ -63,7 +63,7 @@ static test_return_t GDB_COMMAND_test(void *)
 
 static test_return_t test_success_equals_one_test(void *)
 {
-  test_skip(HAVE_LIBMEMCACHED, true);
+  test_skip(HAVE_LIBMEMCACHED, 1);
 #if defined(HAVE_LIBMEMCACHED) && HAVE_LIBMEMCACHED 
   test_zero(MEMCACHED_SUCCESS);
 #endif
@@ -110,7 +110,7 @@ static test_return_t local_not_test(void *)
   }
 
   // unsetenv() will cause issues with valgrind
-  _compare(__FILE__, __LINE__, __func__, 0, unsetenv("LIBTEST_LOCAL"));
+  _compare(__FILE__, __LINE__, __func__, 0, unsetenv("LIBTEST_LOCAL"), true);
   test_compare(0, unsetenv("LIBTEST_LOCAL"));
   test_false(test_is_local());
 
@@ -228,17 +228,51 @@ static test_return_t _compare_gearman_return_t_test(void *)
 static test_return_t gearmand_cycle_test(void *object)
 {
   server_startup_st *servers= (server_startup_st*)object;
-  test_true(servers);
+  test_true(servers and servers->validate());
 
 #if defined(HAVE_GEARMAND_BINARY) && HAVE_GEARMAND_BINARY
   test_true(has_gearmand_binary());
-#else
-  test_skip(true, has_gearmand_binary());
 #endif
 
-  const char *argv[1]= { "cycle_gearmand" };
-  test_true(server_startup(*servers, "gearmand", 9999, 1, argv));
+  test_skip(true, has_gearmand_binary());
 
+  test_true(server_startup(*servers, "gearmand", get_free_port(), 0, NULL));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t memcached_light_cycle_TEST(void *object)
+{
+  server_startup_st *servers= (server_startup_st*)object;
+  test_true(servers);
+
+  test_skip(true, bool(HAVE_MEMCACHED_LIGHT_BINARY));
+
+  test_true(server_startup(*servers, "memcached-light", get_free_port(), 0, NULL));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t skip_shim(bool a, bool b)
+{
+  test_skip(a, b);
+  return TEST_SUCCESS;
+}
+
+static test_return_t test_skip_true_TEST(void *object)
+{
+  test_compare(true, true);
+  test_compare(false, false);
+  test_compare(TEST_SUCCESS, skip_shim(true, true));
+  test_compare(TEST_SUCCESS, skip_shim(false, false));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t test_skip_false_TEST(void *object)
+{
+  test_compare(TEST_SKIPPED, skip_shim(true, false));
+  test_compare(TEST_SKIPPED, skip_shim(false, true));
   return TEST_SUCCESS;
 }
 
@@ -250,8 +284,7 @@ static test_return_t memcached_cycle_test(void *object)
   if (MEMCACHED_BINARY and HAVE_LIBMEMCACHED) 
   {
     test_true(has_memcached_binary());
-    const char *argv[1]= { "cycle_memcached" };
-    test_true(server_startup(*servers, "memcached", 9998, 1, argv));
+    test_true(server_startup(*servers, "memcached", get_free_port(), 0, NULL));
 
     return TEST_SUCCESS;
   }
@@ -264,13 +297,15 @@ static test_return_t memcached_socket_cycle_test(void *object)
   server_startup_st *servers= (server_startup_st*)object;
   test_true(servers);
 
-  if (MEMCACHED_BINARY and HAVE_LIBMEMCACHED)
+  if (MEMCACHED_BINARY)
   {
-    test_true(has_memcached_binary());
-    const char *argv[1]= { "cycle_memcached" };
-    test_true(servers->start_socket_server("memcached", 9997, 1, argv));
+    if (HAVE_LIBMEMCACHED)
+    {
+      test_true(has_memcached_binary());
+      test_true(servers->start_socket_server("memcached", get_free_port(), 0, NULL));
 
-    return TEST_SUCCESS;
+      return TEST_SUCCESS;
+    }
   }
 
   return TEST_SKIPPED;
@@ -281,18 +316,17 @@ static test_return_t memcached_sasl_test(void *object)
   server_startup_st *servers= (server_startup_st*)object;
   test_true(servers);
 
-  if (getenv("TESTS_ENVIRONMENT"))
-  {
-    return TEST_SKIPPED;
-  }
+  test_skip(false, bool(getenv("TESTS_ENVIRONMENT")));
 
-  if (MEMCACHED_SASL_BINARY and HAVE_LIBMEMCACHED)
+  if (MEMCACHED_SASL_BINARY)
   {
-    test_true(has_memcached_sasl_binary());
-    const char *argv[1]= { "cycle_memcached_sasl" };
-    test_true(server_startup(*servers, "memcached-sasl", 9996, 1, argv));
+    if (HAVE_LIBMEMCACHED)
+    {
+      test_true(has_memcached_sasl_binary());
+      test_true(server_startup(*servers, "memcached-sasl", get_free_port(), 0, NULL));
 
-    return TEST_SUCCESS;
+      return TEST_SUCCESS;
+    }
   }
 
   return TEST_SKIPPED;
@@ -308,6 +342,31 @@ static test_return_t application_true_BINARY(void *)
   return TEST_SUCCESS;
 }
 
+static test_return_t application_gdb_true_BINARY2(void *)
+{
+  test_skip(0, access("/usr/bin/gdb", X_OK ));
+  Application true_app("true");
+  true_app.use_gdb();
+
+  test_compare(Application::SUCCESS, true_app.run());
+  test_compare(Application::SUCCESS, true_app.wait());
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t application_gdb_true_BINARY(void *)
+{
+  test_skip(0, access("/usr/bin/gdb", X_OK ));
+  Application true_app("true");
+  true_app.use_gdb();
+
+  const char *args[]= { "--fubar", 0 };
+  test_compare(Application::SUCCESS, true_app.run(args));
+  test_compare(Application::SUCCESS, true_app.wait());
+
+  return TEST_SUCCESS;
+}
+
 static test_return_t application_true_fubar_BINARY(void *)
 {
   Application true_app("true");
@@ -315,7 +374,27 @@ static test_return_t application_true_fubar_BINARY(void *)
   const char *args[]= { "--fubar", 0 };
   test_compare(Application::SUCCESS, true_app.run(args));
   test_compare(Application::SUCCESS, true_app.wait());
-  test_compare(0, (*true_app.stdout_result()).size());
+  test_compare(0, true_app.stdout_result().size());
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t application_doesnotexist_BINARY(void *)
+{
+  test_skip_valgrind();
+
+  Application true_app("doesnotexist");
+
+  const char *args[]= { "--fubar", 0 };
+#if defined(TARGET_OS_OSX) && TARGET_OS_OSX
+  test_compare(Application::INVALID, true_app.run(args));
+  test_compare(Application::FAILURE, true_app.wait());
+#else
+  test_compare(Application::SUCCESS, true_app.run(args));
+  test_compare(Application::INVALID, true_app.wait());
+#endif
+
+  test_compare(0, true_app.stdout_result().size());
 
   return TEST_SUCCESS;
 }
@@ -327,7 +406,7 @@ static test_return_t application_true_fubar_eq_doh_BINARY(void *)
   const char *args[]= { "--fubar=doh", 0 };
   test_compare(Application::SUCCESS, true_app.run(args));
   test_compare(Application::SUCCESS, true_app.wait());
-  test_compare(0, (*true_app.stdout_result()).size());
+  test_compare(0, true_app.stdout_result().size());
 
   return TEST_SUCCESS;
 }
@@ -340,7 +419,7 @@ static test_return_t application_true_fubar_eq_doh_option_BINARY(void *)
 
   test_compare(Application::SUCCESS, true_app.run());
   test_compare(Application::SUCCESS, true_app.wait());
-  test_compare(0, (*true_app.stdout_result()).size());
+  test_compare(0, true_app.stdout_result().size());
 
   return TEST_SUCCESS;
 }
@@ -378,7 +457,8 @@ static test_return_t TRACE_TEST(void *)
 
 static test_return_t vchar_t_TEST(void *)
 {
-  libtest::vchar_t response(test_literal_param("fubar\n"));
+  libtest::vchar_t response;
+  libtest::make_vector(response, test_literal_param("fubar\n"));
   test_compare(response, response);
 
   return TEST_SUCCESS;
@@ -392,7 +472,8 @@ static test_return_t application_echo_fubar_BINARY(void *)
   test_compare(Application::SUCCESS, true_app.run(args));
   test_compare(Application::SUCCESS, true_app.wait());
 
-  libtest::vchar_t response(test_literal_param("fubar\n"));
+  libtest::vchar_t response;
+  make_vector(response, test_literal_param("fubar\n"));
   test_compare(response, true_app.stdout_result());
 
   return TEST_SUCCESS;
@@ -406,7 +487,8 @@ static test_return_t application_echo_fubar_BINARY2(void *)
 
   test_compare(Application::SUCCESS, true_app.run());
   test_compare(Application::SUCCESS, true_app.wait());
-  libtest::vchar_t response(test_literal_param("fubar\n"));
+  libtest::vchar_t response;
+  make_vector(response, test_literal_param("fubar\n"));
   test_compare(response, true_app.stdout_result());
 
   return TEST_SUCCESS;
@@ -485,14 +567,120 @@ static test_return_t wait_services_BINARY2(void *)
   return TEST_SUCCESS;
 }
 
-static test_return_t application_wait_services_BINARY2(void *)
+static test_return_t wait_services_appliction_TEST(void *)
 {
+  test_skip(0, access("/usr/bin/gdb", X_OK ));
   test_skip(0, access("/etc/services", R_OK ));
 
-  libtest::Application("libtest/wait", true);
-  const char *args[]= { "/etc/services", 0 };
+  libtest::Application wait_app("libtest/wait", true);
+  wait_app.use_gdb();
 
-  test_compare(EXIT_SUCCESS, exec_cmdline("libtest/wait", args, true));
+  const char *args[]= { "/etc/services", 0 };
+  test_compare(Application::SUCCESS, wait_app.run(args));
+  test_compare(Application::SUCCESS, wait_app.wait());
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t gdb_wait_services_appliction_TEST(void *)
+{
+#if defined(TARGET_OS_OSX) && TARGET_OS_OSX
+  test_skip(0, TARGET_OS_OSX);
+#endif
+
+  test_skip(0, access("/usr/bin/gdb", X_OK ));
+  test_skip(0, access("/etc/services", R_OK ));
+
+  libtest::Application wait_app("libtest/wait", true);
+  wait_app.use_gdb();
+
+  const char *args[]= { "/etc/services", 0 };
+  test_compare(Application::SUCCESS, wait_app.run(args));
+  test_compare(Application::SUCCESS, wait_app.wait());
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t gdb_abort_services_appliction_TEST(void *)
+{
+  test_skip(0, access("/usr/bin/gdb", X_OK ));
+
+#if defined(TARGET_OS_OSX) && TARGET_OS_OSX
+  test_skip(0, TARGET_OS_OSX);
+#endif
+
+  libtest::Application abort_app("libtest/abort", true);
+  abort_app.use_gdb();
+
+  test_compare(Application::SUCCESS, abort_app.run());
+  test_compare(Application::SUCCESS, abort_app.wait());
+
+  std::string gdb_filename= abort_app.gdb_filename();
+  const char *args[]= { "SIGABRT", gdb_filename.c_str(), 0 };
+  test_compare(EXIT_SUCCESS, exec_cmdline("grep", args));
+
+  // Sanity test
+  args[0]= "THIS_WILL_NOT_BE_FOUND";
+  test_compare(EXIT_FAILURE, exec_cmdline("grep", args));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t get_free_port_TEST(void *)
+{
+  in_port_t ret_port;
+  test_true_hint((ret_port= get_free_port()), ret_port);
+  test_true(get_free_port() != default_port());
+  test_true(get_free_port() != get_free_port());
+
+  return TEST_SUCCESS;
+}
+
+static uint32_t fatal_calls= 0;
+
+static test_return_t fatal_TEST(void *)
+{
+  test_compare(fatal_calls++, fatal::disabled_counter());
+  throw libtest::fatal(LIBYATL_DEFAULT_PARAM, "Testing va_args based fatal(): %d", 10); 
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t number_of_cpus_TEST(void *)
+{
+  test_true(number_of_cpus() >= 1);
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t create_tmpfile_TEST(void *)
+{
+  std::string tmp= create_tmpfile(__func__);
+
+  Application touch_app("touch");
+  const char *args[]= { tmp.c_str(), 0 };
+  test_compare(Application::SUCCESS, touch_app.run(args));
+  test_compare(Application::SUCCESS, touch_app.wait());
+
+  test_compare_hint(0, access(tmp.c_str(), R_OK), strerror(errno));
+  test_compare_hint(0, unlink(tmp.c_str()), strerror(errno));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t fatal_message_TEST(void *)
+{
+  test_compare(fatal_calls++, fatal::disabled_counter());
+  throw fatal_message("Fatal test");
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t default_port_TEST(void *)
+{
+  in_port_t ret_port= default_port();
+  test_compare(ret_port, libtest::default_port());
+  test_compare(ret_port, libtest::default_port());
 
   return TEST_SUCCESS;
 }
@@ -523,12 +711,19 @@ static test_return_t check_for_libmemcached(void *)
 
 test_st memcached_tests[] ={
   {"memcached startup-shutdown", 0, memcached_cycle_test },
+  {"memcached-light startup-shutdown", 0, memcached_light_cycle_TEST },
   {"memcached(socket file) startup-shutdown", 0, memcached_socket_cycle_test },
   {"memcached_sasl() startup-shutdown", 0, memcached_sasl_test },
   {"_compare(memcached_return_t)", 0, _compare_memcached_return_t_test },
   {0, 0, 0}
 };
 
+test_st test_skip_TESTS[] ={
+  {"true, true", 0, test_skip_true_TEST },
+  {"true, false", 0, test_skip_false_TEST },
+  {0, 0, 0}
+};
+
 test_st environment_tests[] ={
   {"LIBTOOL_COMMAND", 0, LIBTOOL_COMMAND_test },
   {"VALGRIND_COMMAND", 0, VALGRIND_COMMAND_test },
@@ -578,13 +773,41 @@ test_st cmdline_tests[] ={
   {"wait --quiet --version", 0, wait_version_BINARY },
   {"wait --quiet /etc/services", 0, wait_services_BINARY },
   {"wait /etc/services", 0, wait_services_BINARY2 },
+  {"wait /etc/services", 0, wait_services_appliction_TEST },
+  {"gdb wait /etc/services", 0, gdb_wait_services_appliction_TEST },
+  {"gdb abort", 0, gdb_abort_services_appliction_TEST },
+  {0, 0, 0}
+};
+
+test_st get_free_port_TESTS[] ={
+  {"get_free_port()", 0, get_free_port_TEST },
+  {"default_port()", 0, default_port_TEST },
+  {0, 0, 0}
+};
+
+test_st fatal_message_TESTS[] ={
+  {"libtest::fatal", 0, fatal_TEST },
+  {"fatal_message()", 0, fatal_message_TEST },
+  {0, 0, 0}
+};
+
+test_st number_of_cpus_TESTS[] ={
+  {"libtest::number_of_cpus()", 0, number_of_cpus_TEST },
+  {0, 0, 0}
+};
+
+test_st create_tmpfile_TESTS[] ={
+  {"libtest::create_tmpfile()", 0, create_tmpfile_TEST },
   {0, 0, 0}
 };
 
 test_st application_tests[] ={
   {"vchar_t", 0, vchar_t_TEST },
   {"true", 0, application_true_BINARY },
+  {"gbd true --fubar", 0, application_gdb_true_BINARY },
+  {"gbd true", 0, application_gdb_true_BINARY2 },
   {"true --fubar", 0, application_true_fubar_BINARY },
+  {"doesnotexist --fubar", 0, application_doesnotexist_BINARY },
   {"true --fubar=doh", 0, application_true_fubar_eq_doh_BINARY },
   {"true --fubar=doh add_option()", 0, application_true_fubar_eq_doh_option_BINARY },
   {"echo fubar", 0, application_echo_fubar_BINARY },
@@ -598,6 +821,18 @@ static test_return_t check_for_curl(void *)
   return TEST_SUCCESS;
 }
 
+static test_return_t disable_fatal_exception(void *)
+{
+  fatal::disable();
+  return TEST_SUCCESS;
+}
+
+static test_return_t enable_fatal_exception(void *)
+{
+  fatal::disable();
+  return TEST_SUCCESS;
+}
+
 test_st http_tests[] ={
   {"GET", 0, GET_TEST },
   {"POST", 0, POST_TEST },
@@ -608,6 +843,7 @@ test_st http_tests[] ={
 collection_st collection[] ={
   {"environment", 0, 0, environment_tests},
   {"return values", 0, 0, tests_log},
+  {"test_skip()", 0, 0, test_skip_TESTS },
   {"local", 0, 0, local_log},
   {"directories", 0, 0, directories_tests},
   {"comparison", 0, 0, comparison_tests},
@@ -616,6 +852,10 @@ collection_st collection[] ={
   {"cmdline", 0, 0, cmdline_tests},
   {"application", 0, 0, application_tests},
   {"http", check_for_curl, 0, http_tests},
+  {"get_free_port()", 0, 0, get_free_port_TESTS },
+  {"fatal", disable_fatal_exception, enable_fatal_exception, fatal_message_TESTS },
+  {"number_of_cpus()", 0, 0, number_of_cpus_TESTS },
+  {"create_tmpfile()", 0, 0, create_tmpfile_TESTS },
   {0, 0, 0, 0}
 };
 
