@@ -50,8 +50,6 @@ using namespace libtest;
 #include <vector>
 #include <boost/foreach.hpp>
 
-#include <tests/ports.h>
-
 #define GEARMAN_CORE
 #include <libgearman/gearman.h>
 
@@ -228,7 +226,7 @@ extern "C"
       pthread_exit(0);
     }
 
-    gearman_return_t rc= gearman_client_add_server(&client, NULL, CLIENT_TEST_PORT);
+    gearman_return_t rc= gearman_client_add_server(&client, NULL, libtest::default_port());
     if (gearman_failed(rc))
     {
       *ret= rc;
@@ -794,19 +792,18 @@ static test_return_t background_failure_test(void *object)
 
 static test_return_t add_servers_test(void *)
 {
-  gearman_client_st client, *client_ptr;
+  gearman_client_st *client= gearman_client_create(NULL);
+  test_truth(client);
 
-  client_ptr= gearman_client_create(&client);
-  test_truth(client_ptr);
+  test_compare_got(GEARMAN_SUCCESS,
+                   gearman_client_add_servers(client, "localhost:4730,localhost"),
+                   gearman_client_error(client));
 
-  gearman_return_t rc;
-  rc= gearman_client_add_servers(&client, "localhost:4730,localhost");
-  test_compare_got(GEARMAN_SUCCESS, rc, gearman_strerror(rc));
+  test_compare_got(GEARMAN_GETADDRINFO,
+                   gearman_client_add_servers(client, "exist.gearman.info:7003,does_not_exist.gearman.info:12345"),
+                   gearman_client_error(client));
 
-  rc= gearman_client_add_servers(&client, "old_jobserver:7003,broken:12345");
-  test_compare_got(GEARMAN_GETADDRINFO, rc, gearman_strerror(rc));
-
-  gearman_client_free(&client);
+  gearman_client_free(client);
 
   return TEST_SUCCESS;
 }
@@ -862,7 +859,7 @@ static test_return_t bug_518512_test(void *)
   test_truth(gearman_client_create(&client));
 
   test_compare(GEARMAN_SUCCESS,
-               gearman_client_add_server(&client, NULL, CLIENT_TEST_PORT));
+               gearman_client_add_server(&client, NULL, libtest::default_port()));
 
   gearman_client_set_timeout(&client, 0);
   gearman_return_t rc;
@@ -873,7 +870,7 @@ static test_return_t bug_518512_test(void *)
   test_zero(result_size);
 
   gearman_function_t func_arg= gearman_function_create_v1(client_test_temp_worker);
-  struct worker_handle_st *completion_worker= test_worker_start(CLIENT_TEST_PORT, NULL, "client_test_temp",
+  struct worker_handle_st *completion_worker= test_worker_start(libtest::default_port(), NULL, "client_test_temp",
                                                                 func_arg, NULL, gearman_worker_options_t());
 
   test_true(completion_worker);
@@ -901,7 +898,7 @@ static test_return_t loop_test(void *)
   gearman_function_t func_arg= gearman_function_create_v1(client_test_temp_worker);
   for (size_t x= 0; x < NUMBER_OF_WORKERS; x++)
   {
-    handles[x]= test_worker_start(CLIENT_TEST_PORT, NULL, "client_test_temp",
+    handles[x]= test_worker_start(libtest::default_port(), NULL, "client_test_temp",
                                   func_arg, NULL, gearman_worker_options_t());
   }
 
@@ -938,7 +935,7 @@ static test_return_t regression_785203_do_test(void *object)
   test_truth(client= gearman_client_create(NULL));
 
   test_compare(GEARMAN_SUCCESS,
-               gearman_client_add_server(client, NULL, CLIENT_TEST_PORT));
+               gearman_client_add_server(client, NULL, libtest::default_port()));
 
   gearman_client_add_options(client, GEARMAN_CLIENT_FREE_TASKS);
   { // All defaults, except timeout_return
@@ -974,7 +971,7 @@ static test_return_t regression_785203_do_background_test(void *object)
   test_truth(client= gearman_client_create(NULL));
 
   test_compare(GEARMAN_SUCCESS,
-               gearman_client_add_server(client, NULL, CLIENT_TEST_PORT));
+               gearman_client_add_server(client, NULL, libtest::default_port()));
 
   gearman_client_add_options(client, GEARMAN_CLIENT_FREE_TASKS);
   { // All defaults, except timeout_return
@@ -1143,6 +1140,16 @@ static test_return_t regression_833394_test(void *object)
   return TEST_SUCCESS;
 }
 
+static test_return_t gearman_client_set_identifier_TEST(void* object)
+{
+  gearman_client_st *client= (gearman_client_st *)object;
+
+  test_compare(GEARMAN_SUCCESS,
+               gearman_client_set_identifier(client, test_literal_param(__func__)));
+
+  return TEST_SUCCESS;
+}
+
 static uint32_t global_counter;
 
 static test_return_t pre_chunk(void *object)
@@ -1250,8 +1257,7 @@ void *client_test_temp_worker(gearman_job_st *, void *,
 
 static void *world_create(server_startup_st& servers, test_return_t& error)
 {
-  const char *argv[1]= { "client_gearmand" };
-  if (server_startup(servers, "gearmand", CLIENT_TEST_PORT, 1, argv) == false)
+  if (server_startup(servers, "gearmand", libtest::default_port(), 0, NULL) == false)
   {
     error= TEST_FAILURE;
     return NULL;
@@ -1260,44 +1266,38 @@ static void *world_create(server_startup_st& servers, test_return_t& error)
 
   client_test_st *test= new client_test_st();
 
-  if (not test)
-  {
-    error= TEST_MEMORY_ALLOCATION_FAILURE;
-    return NULL;
-  }
-
   // Version 1 functions
   gearman_function_t echo_react_fn_v1= gearman_function_create_v1(echo_or_react_worker);
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, WORKER_FUNCTION_NAME, echo_react_fn_v1, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, WORKER_FUNCTION_NAME, echo_react_fn_v1, NULL, gearman_worker_options_t()));
   gearman_function_t echo_react_chunk_fn_v1= gearman_function_create_v1(echo_or_react_chunk_worker);
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, WORKER_CHUNKED_FUNCTION_NAME, echo_react_chunk_fn_v1, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, WORKER_CHUNKED_FUNCTION_NAME, echo_react_chunk_fn_v1, NULL, gearman_worker_options_t()));
 
   // Version 2 functsions
   gearman_function_t echo_react_fn_v2= gearman_function_create(echo_or_react_worker_v2);
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, WORKER_FUNCTION_NAME"_v2", echo_react_fn_v2, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, WORKER_FUNCTION_NAME"_v2", echo_react_fn_v2, NULL, gearman_worker_options_t()));
   gearman_function_t echo_react_chunk_fn_v2= gearman_function_create(echo_or_react_chunk_worker_v2);
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, WORKER_CHUNKED_FUNCTION_NAME"_v2", echo_react_chunk_fn_v2, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, WORKER_CHUNKED_FUNCTION_NAME"_v2", echo_react_chunk_fn_v2, NULL, gearman_worker_options_t()));
 
   gearman_function_t unique_worker_arg= gearman_function_create_v1(unique_worker);
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, WORKER_UNIQUE_FUNCTION_NAME, unique_worker_arg, NULL, GEARMAN_WORKER_GRAB_UNIQ));
+  test->push(test_worker_start(libtest::default_port(), NULL, WORKER_UNIQUE_FUNCTION_NAME, unique_worker_arg, NULL, GEARMAN_WORKER_GRAB_UNIQ));
 
   gearman_function_t split_worker_fn= gearman_function_create_partition(split_worker, cat_aggregator_fn);
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, WORKER_SPLIT_FUNCTION_NAME, split_worker_fn,  NULL, GEARMAN_WORKER_GRAB_ALL));
+  test->push(test_worker_start(libtest::default_port(), NULL, WORKER_SPLIT_FUNCTION_NAME, split_worker_fn,  NULL, GEARMAN_WORKER_GRAB_ALL));
 
   // Namespace versions of the above
-  test->push(test_worker_start(CLIENT_TEST_PORT, NAMESPACE_KEY, WORKER_FUNCTION_NAME, echo_react_fn_v1, NULL, gearman_worker_options_t()));
-  test->push(test_worker_start(CLIENT_TEST_PORT, NAMESPACE_KEY, WORKER_CHUNKED_FUNCTION_NAME, echo_react_chunk_fn_v1, NULL, gearman_worker_options_t()));
-  test->push(test_worker_start(CLIENT_TEST_PORT, NAMESPACE_KEY, WORKER_SPLIT_FUNCTION_NAME, split_worker_fn,  NULL, GEARMAN_WORKER_GRAB_ALL));
+  test->push(test_worker_start(libtest::default_port(), NAMESPACE_KEY, WORKER_FUNCTION_NAME, echo_react_fn_v1, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NAMESPACE_KEY, WORKER_CHUNKED_FUNCTION_NAME, echo_react_chunk_fn_v1, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NAMESPACE_KEY, WORKER_SPLIT_FUNCTION_NAME, split_worker_fn,  NULL, GEARMAN_WORKER_GRAB_ALL));
 
   // Long function name
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, REGRESSION_FUNCTION_833394_54, echo_react_fn_v2, NULL, gearman_worker_options_t()));
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, REGRESSION_FUNCTION_833394_55, echo_react_fn_v2, NULL, gearman_worker_options_t()));
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, REGRESSION_FUNCTION_833394_65, echo_react_fn_v2, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, REGRESSION_FUNCTION_833394_54, echo_react_fn_v2, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, REGRESSION_FUNCTION_833394_55, echo_react_fn_v2, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, REGRESSION_FUNCTION_833394_65, echo_react_fn_v2, NULL, gearman_worker_options_t()));
 
   gearman_function_t increment_reset_worker_fn= gearman_function_create_v1(increment_reset_worker);
   for (uint32_t x= 0; x < 10; x++)
   {
-    test->push(test_worker_start(CLIENT_TEST_PORT, 
+    test->push(test_worker_start(libtest::default_port(), 
                                  NULL,
                                  "increment_reset_worker", increment_reset_worker_fn, 
                                  NULL, gearman_worker_options_t()));
@@ -1305,9 +1305,9 @@ static void *world_create(server_startup_st& servers, test_return_t& error)
 
   // Count worker
   gearman_function_t count_worker_fn= gearman_function_create(count_worker);
-  test->push(test_worker_start(CLIENT_TEST_PORT, NULL, "count", count_worker_fn, NULL, gearman_worker_options_t()));
+  test->push(test_worker_start(libtest::default_port(), NULL, "count", count_worker_fn, NULL, gearman_worker_options_t()));
 
-  if (gearman_failed(gearman_client_add_server(test->client(), NULL, CLIENT_TEST_PORT)))
+  if (gearman_failed(gearman_client_add_server(test->client(), NULL, libtest::default_port())))
   {
     error= TEST_FAILURE;
     return NULL;
@@ -1327,6 +1327,10 @@ static bool world_destroy(void *object)
   return TEST_SUCCESS;
 }
 
+test_st gearman_client_set_identifier_TESTS[] ={
+  {"gearman_client_set_identifier()", 0, gearman_client_set_identifier_TEST },
+  {0, 0, 0}
+};
 
 test_st gearman_id_t_TESTS[] ={
   {"gearman_client_st", 0, gearman_client_st_id_t_TEST },
@@ -1334,7 +1338,7 @@ test_st gearman_id_t_TESTS[] ={
   {0, 0, 0}
 };
 
-test_st tests[] ={
+test_st gearman_client_st_TESTS[] ={
   {"bug_518512_test", 0, bug_518512_test },
   {"init", 0, init_test },
   {"allocation", 0, allocation_test },
@@ -1472,8 +1476,8 @@ test_st limit_tests[] ={
 
 collection_st collection[] ={
   {"gearman_id_t", 0, 0, gearman_id_t_TESTS},
-  {"gearman_client_st", 0, 0, tests},
-  {"gearman_client_st chunky", pre_chunk, post_function_reset, tests}, // Test with a worker that will respond in part
+  {"gearman_client_st", 0, 0, gearman_client_st_TESTS},
+  {"gearman_client_st chunky", pre_chunk, post_function_reset, gearman_client_st_TESTS}, // Test with a worker that will respond in part
   {"gearman_strerror()", 0, 0, gearman_strerror_tests},
   {"gearman_task_add_task()", 0, 0, gearman_task_tests},
   {"gearman_task_add_task() v2 workers", pre_v2, post_function_reset, gearman_task_tests},
@@ -1521,6 +1525,8 @@ static test_return_t _runner_prepost_default(libgearman_test_prepost_callback_fn
 
 static test_return_t _runner_default(libgearman_test_callback_fn func, client_test_st *container)
 {
+  test_compare(GEARMAN_SUCCESS, gearman_client_echo(container->client(), test_literal_param("check")));
+
   if (func)
   {
     test_return_t rc;
