@@ -22,8 +22,6 @@ using namespace libtest;
 #include <tests/basic.h>
 #include <tests/context.h>
 
-#include <tests/ports.h>
-
 #ifndef __INTEL_COMPILER
 #pragma GCC diagnostic ignored "-Wold-style-cast"
 #endif
@@ -42,7 +40,7 @@ static test_return_t gearmand_basic_option_test(void *)
 
 static test_return_t collection_init(void *object)
 {
-  const char *argv[]= { "test_gearmand", 
+  const char *argv[]= {
     "--libmemcached-servers=localhost:12555", 
     "--queue-type=libmemcached",
     0 };
@@ -50,7 +48,7 @@ static test_return_t collection_init(void *object)
   Context *test= (Context *)object;
   assert(test);
 
-  test_truth(test->initialize(3, argv));
+  test_truth(test->initialize(2, argv));
 
   return TEST_SUCCESS;
 }
@@ -72,20 +70,13 @@ static void *world_create(server_startup_st& servers, test_return_t& error)
     return NULL;
   }
 
-  if (server_startup(servers, "memcached", 12555, 0, NULL) == false)
+  if (server_startup(servers, "memcached", default_port(), 0, NULL) == false)
   {
     error= TEST_FAILURE;
     return NULL;
   }
 
-  Context *test= new Context(MEMCACHED_TEST_PORT, servers);
-  if (not test)
-  {
-    error= TEST_MEMORY_ALLOCATION_FAILURE;
-    return NULL;
-  }
-
-  return test;
+  return new Context(default_port(), servers);
 }
 
 static bool world_destroy(void *object)
